/*******************************************************************************
**                                                                            **
**  File:        util.c                                                       **
**  Description: This file provides general routines for The WESP suite       **
**  Author:      Derick Rethans                                               **
**  Date:        1999-06-02                                                   **
**  Version:     1.0                                                          **
**  Lines:       235                                                          **
**                                                                            **
*******************************************************************************/

#include <stdarg.h>
#include <stdlib.h>
#include "c.h"
#include "interface.h"

void ConvertToHex (char* cOut, const u_int8_t uiValue);
char * copy_argv(register char **argv);
void DBToString (char * cSetting, char * row, int iTotalChars);
void error (const char *sFmt, ...);
void FormatTimeStamp (char* cOut, const struct tm tvTime, const u_int32_t iMicrosecs);
bool isHex(char *cHex);
void PrintChar (int iMask, char * cSetting, int iTotalChars);
void PrintEthernetAddr (const char* cAddress);
void PrintIPAddr (const char* cAddress);
void PrintIPXAddr (const char* cAddress);
void PrintWithCare (int iMask, const char *sFmt, ...);
void PrintTimestamp (const char* cTijd);
void warning(const char *sFmt, ...);

/************* FUNCTIONS ******************************************************/

void ConvertToHex (char* cOut, const u_int8_t uiValue)
{
    char cTrans[17] = "0123456789ABCDEF";
    
    cOut[1] = cTrans[uiValue % 16];
    cOut[0] = cTrans[uiValue / 16];
}

char * copy_argv (register char **argv)
{
    char **ppcP;
    u_int uiLen = 0;
    char *pcBuf;
    char *pcSrc, *pcDst;

    ppcP = argv;
    if (*ppcP == 0)
        return 0;
    while (*ppcP)
        uiLen += strlen (*ppcP++) + 1;

    pcBuf = (char *) malloc (uiLen);
    if (pcBuf == NULL)
        error ("copy_argv: malloc");

    ppcP = argv;
    pcDst = pcBuf;
    while ((pcSrc = *ppcP++) != NULL)
    {
        while ((*pcDst++ = *pcSrc++) != '\0');
        pcDst[-1] = ' ';
    }
    pcDst[-1] = '\0';

    return pcBuf;
}

void DBToString (char * cSetting, char * row, int iTotalChars)
{
    int iI;

    for (iI = 0; iI < iTotalChars; iI++)
    {
        cSetting[iI] = row[iI];
    }
    if (row[1] == 0)
    {
        cSetting[1] = cSetting[0];
        cSetting[0] = '0';
    }
}

void error (const char *sFmt, ...)
{
    va_list vlAp;

    (void) fprintf (stderr, "The WESP: ");
    va_start (vlAp, sFmt);
    (void) vfprintf (stderr, sFmt, vlAp);
    va_end (ap);
    if (*sFmt)
    {
        sFmt += strlen (sFmt);
        if (sFmt[-1] != '\n')
            (void) fputc ('\n', stderr);
    }
    exit (1);
}

void FormatTimeStamp (char* cOut, const struct tm tvTime, const u_int32_t iMicrosecs)
{
    sprintf (cOut, "%04d%02d%02d%02d%02d%02d%06u", tvTime.tm_year + 1900,
                                                   tvTime.tm_mon + 1,
			                           tvTime.tm_mday,
		                                   tvTime.tm_hour,
	         				   tvTime.tm_min,
						   tvTime.tm_sec,
						   iMicrosecs);
}

bool isHex(char *cHex)
{
    int iTeller;
    int iMax;
    bool  bBool;

    iMax = strlen(cHex);

    for (iTeller = 0 ; iTeller < iMax; iTeller++)
    {
        bBool = false;
        if ((cHex[iTeller] >= '0' && cHex[iTeller] <= '9') ||
            (cHex[iTeller] >= 'A' && cHex[iTeller] <= 'F') ||
            (cHex[iTeller] >= 'a' && cHex[iTeller] <= 'f'))
            bBool = true;
        if (bBool == false)
            return false;
    }
    return true;
}

void PrintChar (int iMask, char * cSetting, int iTotalChars)
{
    int iI;

    for (iI = 0; iI < iTotalChars ;iI++)
    {
        PrintWithCare(iMask, "%c", cSetting[iI]);
    }
}

void PrintEthernetAddr (const char* cAddress)
{
    printf ("%c%c:%c%c:%c%c:%c%c:%c%c:%c%c",
             cAddress[0],  cAddress[1],  cAddress[2],
             cAddress[3],  cAddress[4],  cAddress[5],
             cAddress[6],  cAddress[7],  cAddress[8],
             cAddress[9],  cAddress[10], cAddress[11]);
}

void PrintIPAddr (const char* cAddress)
{
    char cHelp[3];

    cHelp[2] = 0;

    cHelp[0] = cAddress[0];
    cHelp[1] = cAddress[1];
    printf ("%ld.", strtol (cHelp, 0, 16));
    cHelp[0] = cAddress[2];
    cHelp[1] = cAddress[3];
    printf ("%ld.", strtol (cHelp, 0, 16));
    cHelp[0] = cAddress[4];
    cHelp[1] = cAddress[5];
    printf ("%ld.", strtol (cHelp, 0, 16));
    cHelp[0] = cAddress[6];
    cHelp[1] = cAddress[7];
    printf ("%ld", strtol (cHelp, 0, 16));
}

void PrintIPXAddr (const char* cAddress)
{
    char cHelp[5];

    cHelp[4] = 0;
    cHelp[0] = cAddress[20];
    cHelp[1] = cAddress[21];
    cHelp[2] = cAddress[22];
    cHelp[3] = cAddress[23];

    printf ("%c%c%c%c%c%c%c%c %c%c-%c%c-%c%c-%c%c-%c%c-%c%c %ld",
             cAddress[0],  cAddress[1],  cAddress[2],
             cAddress[3],  cAddress[4],  cAddress[5],
             cAddress[6],  cAddress[7],  cAddress[8],
             cAddress[9],  cAddress[10], cAddress[11],
             cAddress[12], cAddress[13], cAddress[14],
             cAddress[15], cAddress[16], cAddress[17],
             cAddress[18], cAddress[19],
             strtol (cHelp, 0, 16));
}

void PrintWithCare (int iMask, const char *sFmt, ...)
{
    va_list vlAp;
    
    if (iMask < DONT_PRINT)
    {
        va_start (vlAp, sFmt);
        (void) vfprintf (stdout, sFmt, vlAp);
        va_end (vlAp);
    }
}

void PrintTimestamp (const char* cTijd)
{
    char sTimeStamp[21];
    int iT = 0;

    for (iT = 0; iT < 20; iT++)
    {
        sTimeStamp[iT] = cTijd[iT];
    }
    sTimeStamp[20] = 0;
    printf ("%s", sTimeStamp);
}

void warning(const char *sFmt, ...)
{
    va_list vlAp;

    (void) fprintf (stderr, "wesp: WARNING: ");
    va_start (vlAp, sFmt);
    (void) vfprintf (stderr, sFmt, vlAp);
    va_end (vlAp);
    if (*sFmt)
    {
        sFmt += strlen(sFmt);
        if (sFmt[-1] != '\n')
            (void) fputc ('\n', stderr);
    }
}

/* EOF */