/*******************************************************************************
**                                                                            **
**  File:        packet.c                                                     **
**  Description: Routines that parse the packet into a structure.             **
**  Author:      Derick Rethans                                               **
**  Date:        1999-06-02                                                   **
**  Version:     1.0pre3                                                      **
**  Lines:       381                                                          **
**                                                                            **
*******************************************************************************/

#include "util.h"
#include "packet.h"
#include <stdlib.h>
#include <string.h>

struct tICMPTypes acICMPTypes[] =
    {
        {0, "Echo Reply"},
	{3, "Destinaction Unreachable"},
	{4, "Search Quench"},
        {5, "Redirect"},
	{8, "Echo Request"},
	{11, "Time Exceeded"},
	{12, "Parameter Problem"},
	{13, "Timestamp"},
	{14, "Timestamp Reply"},
	{15, "Information Request"},
	{16, "Information Reply"},
        {98, "Fragmented"},
	{99,  ""}
    };

static void GiveEthernetAddr (char * cReturn, const u_char * cData, int iPosition);
static void GiveIPAddr (char * cReturn, const u_char * cData, int iPosition);
static void GiveIPXAddr (char * cReturn, const u_char * cData, int iPosition);
static bit ReturnBit (const u_char* data, const int iPosition, const int iBitPos);
static u_int8_t Return4 (const u_char* data, const int iPosition, const int iOneOrTwo);
static u_int8_t Return8 (const u_char* data, const int iPosition);
static u_int16_t Return16 (const u_char* data, const int iPosition);
static u_int32_t Return32 (const u_char* data, const int iPosition);

/************* FUNCTIONS ******************************************************/

static void GiveEthernetAddr (char * cReturn, const u_char * cData, int iPosition)
{
    ConvertToHex (cReturn+ 0, cData[iPosition+0]);
    ConvertToHex (cReturn+ 2, cData[iPosition+1]);
    ConvertToHex (cReturn+ 4, cData[iPosition+2]);
    ConvertToHex (cReturn+ 6, cData[iPosition+3]);
    ConvertToHex (cReturn+ 8, cData[iPosition+4]);
    ConvertToHex (cReturn+10, cData[iPosition+5]);
}

static void GiveIPAddr (char * cReturn, const u_char * cData, int iPosition)
{
    ConvertToHex (cReturn+ 0, cData[iPosition+0]);
    ConvertToHex (cReturn+ 2, cData[iPosition+1]);
    ConvertToHex (cReturn+ 4, cData[iPosition+2]);
    ConvertToHex (cReturn+ 6, cData[iPosition+3]);
}

static void GiveIPXAddr (char * cReturn, const u_char * cData, int iPosition)
{
    ConvertToHex (cReturn+ 0, cData[iPosition+ 0]);
    ConvertToHex (cReturn+ 2, cData[iPosition+ 1]);
    ConvertToHex (cReturn+ 4, cData[iPosition+ 2]);
    ConvertToHex (cReturn+ 6, cData[iPosition+ 3]);
    ConvertToHex (cReturn+ 8, cData[iPosition+ 4]);
    ConvertToHex (cReturn+10, cData[iPosition+ 5]);
    ConvertToHex (cReturn+12, cData[iPosition+ 6]);
    ConvertToHex (cReturn+14, cData[iPosition+ 7]);
    ConvertToHex (cReturn+16, cData[iPosition+ 8]);
    ConvertToHex (cReturn+18, cData[iPosition+ 9]);
    ConvertToHex (cReturn+20, cData[iPosition+10]);
    ConvertToHex (cReturn+22, cData[iPosition+11]);
}

static bit ReturnBit (const u_char* data, const int iPosition, const int iBitPos)
{
    return (data[iPosition] >> iBitPos) & 0x01;
}

static u_int8_t Return4 (const u_char* data, const int iPosition,
                         const int iOneOrTwo)
{
    if (iOneOrTwo == 1)
    {
        return data[iPosition] >> 4 & 0x0F;
    }
    else
    {
        return data[iPosition] & 0x0F;
    } 
}

static u_int8_t Return8 (const u_char* data, const int iPosition)
{
    return (u_int8_t) data[iPosition];
}

static u_int16_t Return16 (const u_char* data, const int iPosition)
{
    return (u_int8_t) data[iPosition] * 256 + (u_int8_t) data[iPosition+1];
}

static u_int32_t Return32 (const u_char* data, const int iPosition)
{
    return (u_int8_t) data[iPosition] * 16777216 + (u_int8_t) data[iPosition+1] * 65536 +
           (u_int8_t) data[iPosition+2] * 256 + (u_int8_t) data[iPosition+3];
}

void ParsePacket (const u_char* pack, packet* pakketje, const u_int uLLC)
{
    int    iProtocol =        P2N_UNKN;
    int    iProtocol2 =       P3N_UNKN;
    int    iT;
    char  *pData;
    u_int  iTCPDataLength;
    int    iMACHeaderLength = 0;
    char  *pH;
    
    pakketje->iPacketLength = 0;
    pakketje->iOSILayer2Prot = P2N_UNKN; 
    pakketje->iOSILayer3Prot = P3N_UNKN; 
    pakketje->iOSILayer4Prot = P4N_UNKN; 
    pakketje->uiDataLength = 0;
    strcpy (pakketje->cData, "");

    switch (uLLC)
    {
        case P2N_RAW:
	{
            pakketje->iOSILayer2Prot = P2N_RAW; 
    
            GiveEthernetAddr (pakketje->eaEthDestination.addr, "\0\0\0\0\0\0", 0);
            GiveEthernetAddr (pakketje->eaEthSource.addr, "\0\0\0\0\0\0", 0);

	    for (iT = 0; iT < 12; iT++)
	         pakketje->eaEthSource.addr[iT] =  '0';
	    for (iT = 0; iT < 12; iT++)
	         pakketje->eaEthSource.addr[iT] =  '0';
	    iProtocol = P3N_IP;
            pakketje->iEthernetProtocol = iProtocol;
	}
	break;
        case P2N_LO:
	{
            pakketje->iOSILayer2Prot = P2N_LO; 
    
            GiveEthernetAddr (pakketje->eaEthDestination.addr, "\0\0\0\0\0\0", 0);
            GiveEthernetAddr (pakketje->eaEthSource.addr, "\0\0\0\0\0\0", 0);

	    for (iT = 0; iT < 12; iT++)
	         pakketje->eaEthSource.addr[iT] =  '0';
	    for (iT = 0; iT < 12; iT++)
	         pakketje->eaEthSource.addr[iT] =  '0';
            pakketje->iEthernetProtocol = Return16 (pack, 2);
            switch (pakketje->iEthernetProtocol)
            {
        	case P3N_IP:  iProtocol = P3N_IP;
        	              break;
        	case P3N_ARP: iProtocol = P3N_ARP;
        	              break;
        	default:      iProtocol = P3N_UNKN;
            }
            pack = pack + 4;
	    pakketje->iPacketLength += 4;
	    iMACHeaderLength = 4;
	}
	break;
        case P2N_ETH:
	{
            pakketje->iOSILayer2Prot = P2N_ETH; 
    
            GiveEthernetAddr (pakketje->eaEthDestination.addr, pack, 0);
            GiveEthernetAddr (pakketje->eaEthSource.addr, pack, 6);
            pakketje->iEthernetProtocol = Return16(pack, 12);
	    iMACHeaderLength = ETHERNET_HEADER_LENGTH;
            pakketje->iPacketLength += ETHERNET_HEADER_LENGTH;
    
            if (pakketje->iEthernetProtocol == P3N_IP)
	    {
        	iProtocol = P3N_IP;
                pack = pack + ETHERNET_HEADER_LENGTH;
	    }
	    else if (pakketje->iEthernetProtocol == P3N_ARP)
	    {
        	iProtocol = P3N_ARP;
                pack = pack + ETHERNET_HEADER_LENGTH;
	    }
	    else if (pakketje->iEthernetProtocol == P3N_IPX)
	    {
        	iProtocol = P3N_IPX;
                pack = pack + ETHERNET_HEADER_LENGTH;
		pakketje->uiIPXSubType = P3N_ETHII;
	    }
	    else if (pakketje->iEthernetProtocol < 0x05DC)
	    {
	        if (Return16 (pack, 14) == 0xFFFF)
		{
            	    iProtocol = P3N_IPX;
                    pack = pack + ETHERNET_HEADER_LENGTH;
		    pakketje->iEthernetProtocol = P3N_IPX;
		    pakketje->uiIPXSubType = P3N_ETH8023;
                 }
		 else if (Return16 (pack, 14) == 0xE0E0)
		 {
            	    iProtocol = P3N_IPX;
                    pack = pack + ETHERNET_HEADER_LENGTH + 3;
          	    pakketje->iPacketLength += 3;
		    pakketje->iEthernetProtocol = P3N_IPX;
     		    pakketje->uiIPXSubType = P3N_8023;
                 }
		 else if (Return16 (pack, 14) == 0xAAAA)
		 {
            	    iProtocol = P3N_IPX;
                    pack = pack + ETHERNET_HEADER_LENGTH + 8;
          	    pakketje->iPacketLength += 8;
		    pakketje->iEthernetProtocol = P3N_IPX;
     		    pakketje->uiIPXSubType = P3N_ETHSNAP;
                 }
	    }
	    else
	    {
        	iProtocol = P3N_UNKN;
            }
	}
	break;
    }
    
    switch (iProtocol)
    {
        case P3N_IPX:
	    {
                pakketje->iOSILayer3Prot = P3N_IPX;

                pakketje->uiIPXChecksum = Return16 (pack, 0);
                pakketje->uiIPXLength = Return16 (pack, 2);
                pakketje->uiIPXTransportControl = Return8 (pack, 4);
                pakketje->uiIPXPacketType = Return8 (pack, 5);
                GiveIPXAddr (pakketje->ipaIPXSource.networkaddr, pack,  18);
                GiveIPXAddr (pakketje->ipaIPXDestination.networkaddr, pack, 6);
      		pakketje->iPacketLength += pakketje->uiIPXLength;
		
		pakketje->iOSILayer4Prot = pakketje->uiIPXPacketType;
	    }
	    break;
        case P3N_IP:
	    {
                pakketje->iOSILayer3Prot = P3N_IP;

		pakketje->bIPVersion = Return4 (pack, 0, 1);
		pakketje->bIPHeaderLength = Return4 (pack, 0, 2);
		pakketje->bIPTypeOfService = Return8 (pack, 1);
		pakketje->uiIPTotalLength = (u_int16_t) Return16 (pack, 2);
		pakketje->uiIPID = Return16 (pack, 4);
		pakketje->bIPDontFragment = ReturnBit (pack, 6, 6);
		pakketje->bIPMoreFragments = ReturnBit (pack, 6, 5);
		pakketje->uiIPFragmentOffset = 
		    ReturnBit (pack, 6, 4) * 4096 +
		    ReturnBit (pack, 6, 3) * 2048 +
		    ReturnBit (pack, 6, 2) * 1024 +
		    ReturnBit (pack, 6, 1) *  512 +
		    ReturnBit (pack, 6, 0) *  256 +
		    Return8 (pack, 7);
		pakketje->uiIPTimeToLive = Return8 (pack, 8);
		pakketje->bIPProtocol = Return8 (pack, 9);
		pakketje->uiIPHeaderChecksum = Return16 (pack, 10);
		GiveIPAddr (pakketje->iaIPSource.addr, pack, 12);
		GiveIPAddr (pakketje->iaIPDestination.addr, pack, 16);
      		pakketje->iPacketLength += pakketje->uiIPTotalLength;
		switch (pakketje->bIPProtocol)
		{
		    case P4N_ICMP:
                        pakketje->iOSILayer4Prot = P4N_ICMP;

                        iProtocol2 = P4N_ICMP;
          		pack = pack + 4 * pakketje->bIPHeaderLength;
                        if (pakketje->uiIPFragmentOffset != 0)
                        {
                            pakketje->uiICMPType = 98; /* Code for fragmented ICMP packets */
                            pakketje->uiICMPCode = 0;
			    pakketje->uiICMPEchoID = 0;
			    pakketje->uiICMPEchoSequenceNr = 0;
                        }
                        else
                        {
          		    pakketje->uiICMPType = Return8 (pack, 0);
          		    pakketje->uiICMPCode = Return8 (pack, 1);
          		    pakketje->uiICMPChecksum = Return16 (pack, 2);
          		    if (pakketje->uiICMPType == 0 || pakketje->uiICMPType == 8)
          		    {
          		        pakketje->uiICMPEchoID = Return8 (pack, 5) * 256 + Return8 (pack, 4);
          		        pakketje->uiICMPEchoSequenceNr = Return8 (pack, 7) * 256 + Return8 (pack, 6);
          		    }
			    else
			    {
			        pakketje->uiICMPEchoID = 0;
			        pakketje->uiICMPEchoSequenceNr = 0;
			    }
                        }
		        break;
		    case P4N_TCP:
                        pakketje->iOSILayer4Prot = P4N_TCP;

                        iProtocol2 = P4N_TCP;
          		pack = pack + 4 * pakketje->bIPHeaderLength;
          		pakketje->uiTCPSourcePort = Return16 (pack, 0);
          		pakketje->uiTCPDestinationPort = Return16 (pack, 2);
          		pakketje->uiTCPSequenceNr = Return32 (pack, 4);
          		pakketje->uiTCPAckNr = Return32 (pack, 8);
          		pakketje->uiTCPHeaderLength = Return4 (pack, 12, 1);
          		pakketje->uiTCPFlags = 
			    ReturnBit (pack, 13, 5) * 32 +
			    ReturnBit (pack, 13, 4) * 16 +
			    ReturnBit (pack, 13, 3) *  8 +
			    ReturnBit (pack, 13, 2) *  4 +
			    ReturnBit (pack, 13, 1) *  2 +
			    ReturnBit (pack, 13, 0);
          		pakketje->uiTCPWindow = Return16 (pack, 14);
          		pakketje->uiTCPChecksum = Return16 (pack, 16);
          		pakketje->uiTCPUrgentPointer = Return16 (pack, 18);
			(u_int32_t) pData = (u_int32_t) pack;
			pData = pData + (Return4 (pack, 12, 1) * 4);
			pakketje->cData[0] = 0;
			iTCPDataLength = pakketje->iPacketLength - iMACHeaderLength - (pakketje->bIPHeaderLength * 4) - (Return4 (pack, 12, 1) * 4);
			if (iTCPDataLength > 254)
			    iTCPDataLength = 254;
		    	memcpy (pakketje->cData, pData, iTCPDataLength);
			pakketje->uiDataLength = iTCPDataLength;
			pakketje->cData[iTCPDataLength] = 0;
			if (iTCPDataLength > 0)
			{
			    if (pakketje->uiTCPSourcePort == 80 || pakketje->uiTCPDestinationPort == 80)
			    {
				if ((pData[0] == 'G' && pData[1] == 'E' && pData[2] == 'T') ||
			    	    (pData[0] == 'H' && pData[1] == 'T' && pData[2] == 'T'))
				{ 
				    pH = strstr (pakketje->cData, "\r\n\r\n");
				    if (pH != NULL) pH[0] = 0;
				}
                    	    }
			}
		        break;
		    case P4N_UDP:
                        pakketje->iOSILayer4Prot = P4N_UDP;

                        iProtocol2 = P4N_UDP;
          		pack = pack + 4 * pakketje->bIPHeaderLength;
          		pakketje->uiUDPSourcePort = Return16 (pack, 0);
          		pakketje->uiUDPDestinationPort = Return16 (pack, 2);
          		pakketje->uiUDPHeaderLength = Return16 (pack, 4);
          		pakketje->uiUDPChecksum = Return16 (pack, 6);
		        break;
		    default:
		}
	    }
	    break;
        case P3N_ARP:
	    {
                pakketje->iOSILayer3Prot = P3N_ARP;

                pakketje->uiARPHardwareType = Return16(pack, 0);
                pakketje->uiARPProtocolType = Return16(pack, 2);
                pakketje->uiARPHardwareLength = Return8(pack, 4);
                pakketje->uiARPProtocolLength = Return8(pack, 5);
                pakketje->uiARPOperation = Return16(pack, 6);
                GiveEthernetAddr (pakketje->eaSender.addr, pack, 8);
                GiveIPAddr (pakketje->iaSender.addr, pack, 14);
                GiveEthernetAddr (pakketje->eaTarget.addr, pack, 18);
                GiveIPAddr (pakketje->iaTarget.addr, pack, 24);
		pakketje->iPacketLength += ARP_HEADER_LENGTH;

                pakketje->iOSILayer4Prot = pakketje->uiARPOperation;
	    }
	    break;
    }
}

/* EOF */