/*******************************************************************************
**                                                                            **
**  File:        pcap_interface.c                                             **
**  Description: The interface to the pcap library.                           **
**  Author:      Derick Rethans / Bjorn Vermeulen                             **
**  Date:        1999-06-02                                                   **
**  Version:     1.0pre3                                                      **
**  Lines:       149                                                          **
**                                                                            **
*******************************************************************************/

#include <pcap.h>
#include <string.h>
#include <unistd.h>
#include "interface.h"
#include "util.h"

int            iSnaplen = DEFAULT_SNAPLEN;
static pcap_t *pd;
pcap_handler   phPrinter;
char          *program_name;
int            iOp;
char          *device;
char         **argvPcap;

struct printer 
{
    pcap_handler phHandle;
    int          iType;
};

static struct printer printers[] = 
{
    { ether_if_print, DLT_EN10MB },    // Ethernet
    { ether_if_print, DLT_IEEE802 },
    { raw_if_print,   DLT_RAW },       // PPP
    { lo_if_print,    DLT_NULL },      // Loopback
    { NULL,           0 },
};

static pcap_handler LookupPrinter (int iType);
void PcapClose (void);
void PcapInit (char **argv, int iOp, char *device);
void PcapInitLib (char **argv, char *device);
void PcapPrintStatus (void);

/************* FUNCTIONS ******************************************************/

static pcap_handler LookupPrinter (int iType)
{
    struct printer *pPrinter;

    for (pPrinter = printers; pPrinter->phHandle; ++pPrinter)
        if (iType == pPrinter->iType)
            return pPrinter->phHandle;

    error("unknown data link type 0x%x", iType);
    /* NOTREACHED */
    return pPrinter->phHandle;      /* To suppress warning */
}

void PcapClose (void)
{
    pcap_close (pd);
}

int PcapDo (void)
{
    return pcap_dispatch (pd, 1, phPrinter, 0);
}

void PcapInit (char **argv, int iOp, char *device)
{
    char     *cp;
    char      ebuf[PCAP_ERRBUF_SIZE];

    if ((cp = strrchr(argv[0], '/')) != NULL)
        program_name = cp + 1;
    else
        program_name = argv[0];

   /* Als er geen interface is opgegeven met -i kies dan de default */
    if (device == NULL)
        device = pcap_lookupdev(ebuf);
    if (device == NULL)
        error("%s", ebuf);
	
   PcapInitLib (argv, device);
}

void PcapInitLib (char **argv, char *device)
{
    int                 iI;
    bpf_u_int32         localnet;
    bpf_u_int32         netmask;
    char               *cmdbuf;
    struct bpf_program  fcode;
    char                ebuf[PCAP_ERRBUF_SIZE];
	
   /* Initilise pcap library */
    pd = pcap_open_live (device, iSnaplen, 1, 100, ebuf);
    if (pd == NULL)
        error("%s", ebuf);
    iI = pcap_snapshot (pd);
    if (iSnaplen < iI)
    {
        warning("iSnaplen raised from %d to %d", iSnaplen, iI);
        iSnaplen = iI;
    }
    if (pcap_lookupnet (device, &localnet, &netmask, ebuf) < 0)
    {
        localnet = 0;
        netmask = 0;
        warning("%s", ebuf);
    }

    setuid (getuid ());

    cmdbuf = copy_argv (&argv[optind]);

    if (pcap_compile (pd, &fcode, cmdbuf, 1, netmask) < 0)
        error ("%s", pcap_geterr (pd));
    if (pcap_setfilter (pd, &fcode) < 0)
        error("%s", pcap_geterr (pd));

    phPrinter = LookupPrinter (pcap_datalink (pd));
    (void) fprintf (stderr, "%s: listening on %s\n", program_name, device);
}

void PcapPrintStatus (void)
{
    struct pcap_stat stat;

    /* Can't print the summary if reading from a savefile */
    if (pd != NULL && pcap_file (pd) == NULL)
    {
        (void) fflush (stdout);
        putc ('\n', stderr);
	if (pcap_stats(pd, &stat) < 0)
            (void) fprintf (stderr, "pcap_stats: %s\n", pcap_geterr (pd));
        else
        {
            (void) fprintf (stderr, "%d packets received by filter\n", stat.ps_recv);
            (void) fprintf (stderr, "%d packets dropped by kernel\n", stat.ps_drop);
	}
    }
}

/* EOF */