/*******************************************************************************
**                                                                            **
**  File:        wesp.c                                                       **
**  Description: Main file of the packet capture program.                     **
**  Author:      Derick Rethans                                               **
**  Date:        1999-06-02                                                   **
**  Version:     1.0pre3                                                      **
**  Lines:       656                                                          **
**                                                                            **
*******************************************************************************/

#include <signal.h>
#include <string.h>
#include <unistd.h>

#include "db_interface.h"
#include "pcap_interface.h"
#include "interface.h"
#include "trigger_inst.h"
#include "filter_inst.h"
#include "packet.h"
#include "util.h"

long int         iDataVerkeer =         0;
bool             bCheckForTrigger =     false;
bool             bCheckForStopTrigger = false;
bool             bTriggered =           false;
bool             bStopTrigger =         false;
char            *program_name =		"The Wesp";
bool             bSleeping =            false;
int              iNrPacketsSinceTrigger;
trigger_setting  tsSetting;
filter_setting   fsSetting;
int              iState =               STATE_PRE;
int              iTotPack =             0;
int              iSnifPack =            0;
int              iFiltPack =            0;
int              iSnifData =            0;
int              iFiltData =            0;
MYSQL * conn;

#define LCK_SET    1
#define LCK_REMOVE 2
#define LCK_CHECK  3

#define LCK_NOT_LOCKED 100
#define LCK_LOCKED     101

static void Calls (int);
static boolean CheckFilter (const struct filter_setting* fsSetting, const struct packet* pPakketje);
static boolean CheckStartTrigger (const struct trigger_setting* tsSetting, const struct packet* pPakketje);
static boolean CheckStopTrigger (const struct trigger_setting* tsSetting, const struct packet* pPakketje);
static void Cleanup (int);
static int Locking (int iOpcode);
static void Reload (int);
static void Usage (void);

/************* FUNCTIONS ******************************************************/

int main (int argc, char **argv)
{
    void (*oldhandler) (int);
    
    argvPcap = argv;
    
    printf ("%s", cASCIIArt);
    printf ("\n        The W.E.S.P. version %s\n\n%s\n\n", version, cCopyright);
    printf ("=================================================================\n");
    conn = InitDBConn ();
	
    device = NULL;

    if (Locking (LCK_CHECK) == LCK_LOCKED)
    {
        printf ("wesp already started or wesp didn't exit correctly the last time\n");
        exit (-2);
    }
	
    opterr = 0;
    while ((iOp = getopt (argc, argv, "i:")) != EOF)
    switch (iOp) 
    {
        case 'i':
            device = optarg;    // specifies the device (eth0/ppp0)
            break;
        default:
            Usage();
    }

    (void) signal (SIGTERM, Cleanup);
    (void) signal (SIGINT, Cleanup);
    (void) signal (SIGUSR1, Reload);
    (void) signal (SIGUSR2, Calls);
    (void) signal (SIGSTOP, Cleanup); 
    (void) signal (SIGQUIT, Cleanup); 

    /* Cooperate with nohup(1) */
    if ((oldhandler = signal (SIGHUP, Cleanup)) != SIG_DFL)
        (void) signal (SIGHUP, oldhandler);
    
    (void) fflush (stderr);
    PcapInit (argv, iOp, device);
    
    (void) Locking (LCK_SET);
    if (!GetTriggerSetting ("default", &tsSetting))
    {
        (void) Locking (LCK_REMOVE);
        exit (-21);
    }
    if (!GetFilterSetting ("default", &fsSetting))
    {
        (void) Locking (LCK_REMOVE);
        exit (-22);
    }
    while (1)
    {
        bCheckForTrigger = true;
        bTriggered = false;
        bStopTrigger = false;
        tsSetting.iStopType != TS_GEEN ? bCheckForStopTrigger = true :
                                         bCheckForStopTrigger;
        printf ("Waiting for trigger...\n");
	iState = STATE_PRE;
        while (!bTriggered)
        {
            PcapDo();
        }

        printf ("\nGot Trigger\n");
	iState = STATE_IN;
        iNrPacketsSinceTrigger = 1;
        bStopTrigger = false;
        bSleeping = true;
        while (!bStopTrigger)
        {
            iNrPacketsSinceTrigger++;
            PcapDo();
        }
        printf ("\nTrigger stopped!\n");
        printf ("Going to sleep...\n");
	iState = STATE_POST;
        while (bSleeping)
        {
            PrintWithCare (4, "Sleeping... \n");
            printf ("z"); fflush (stdout);
            sleep (5);
        }
    }
    exit(0); /* Never reached */
}

static void Calls (int signo)
{
    int iCallID;
    
    iCallID = GetCall();
    switch (iCallID)
    {
        case CALL_STATUS: PutStatusData (iState,
	                                 iTotPack, iSnifPack, iFiltPack, 
	                                 iSnifData, iFiltData);
                          break;
	case CALL_ABORT:  bStopTrigger = true;
			  break;
	case CALL_RELOAD: Reload (SIGUSR2);
			  break;
	default:          break;
    } 
}

static boolean CheckFilter (const struct filter_setting* fsSetting, 
                            const struct packet* pPakketje)
{
    bool bSourceOk;
    bool bDestOk;
    bool bProtOk;
    bool bSourcePortOk;
    bool bDestPortOk;
    bool bSourceSockOk;
    bool bDestSockOk;
    int iReturn = false;
    
    switch (fsSetting->iType)
    {
        case TF_GEEN:
	    iReturn = true;
	    break;
	case TF_ETHERNET:
	    bSourceOk = (strncmp (pPakketje->eaEthSource.addr, fsSetting->eaSource.addr, 12) == 0);
	    bDestOk =   (strncmp (pPakketje->eaEthDestination.addr, fsSetting->eaDestination.addr, 12) == 0);
	    bProtOk =   (pPakketje->iEthernetProtocol == fsSetting->iEthernetProtocol);
            
	    iReturn = true;
	    if (fsSetting->bEthernetProtocolSpec && !bProtOk)
	    {
	        iReturn = false;
	    }
            else
            if (fsSetting->bEthernetSrcSpec && !bSourceOk)
            {
                iReturn = false;
            }
            else
            if (fsSetting->bEthernetDestSpec && !bDestOk)
            {
                iReturn = false;
            }
	    break;
	case TF_IP:
            if (pPakketje->iEthernetProtocol != P3N_IP)
            {
                iReturn = false;
                break;
            }
	    bSourceOk = (strncmp (pPakketje->iaIPSource.addr, fsSetting->iaSourceMask.addr, 8) == 0);
	    bDestOk =   (strncmp (pPakketje->iaIPDestination.addr, fsSetting->iaDestinationMask.addr, 8) == 0);
            bProtOk =   (pPakketje->bIPProtocol == fsSetting->iIPProtocol);
	    if (fsSetting->iIPProtocol == P4N_TCP)
	    {
	        bSourcePortOk = (fsSetting->iIPSrcPort == pPakketje->uiTCPSourcePort);
	        bDestPortOk = (fsSetting->iIPDestPort == pPakketje->uiTCPDestinationPort);
	    }
	    else
	    if (fsSetting->iIPProtocol == P4N_UDP)
	    {
	        bSourcePortOk = (fsSetting->iIPSrcPort == pPakketje->uiUDPSourcePort);
	        bDestPortOk = (fsSetting->iIPDestPort == pPakketje->uiUDPDestinationPort);
	    }
	    else
	    {
	        bSourcePortOk = true;
                bDestPortOk = true;
	    }
	    iReturn = true;
	    if (fsSetting->bIPSrcPortSpec && !bSourcePortOk)
	    {
	        iReturn = false;
	    }
            else
            if (fsSetting->bIPDestPortSpec && !bDestPortOk)
            {
                iReturn = false;
            }
            else
	    if (fsSetting->bIPProtocolSpec && !bProtOk)
	    {
	        iReturn = false;
	    }
            else
            if (fsSetting->bIPSrcSpec && !bSourceOk)
            {
               iReturn = false;
            }
            else
            if (fsSetting->bIPDestSpec && !bDestOk)
            {
                iReturn = false;
            }
	    break;
	case TF_IPX:
            if (pPakketje->iEthernetProtocol != P3N_IPX)
            {
                iReturn = false;
                break;
            }
	    bSourceOk = (strncmp (pPakketje->ipaIPXSource.networkaddr, fsSetting->ipaSource.networkaddr, 8) == 0);
	    bDestOk =   (strncmp (pPakketje->ipaIPXDestination.networkaddr, fsSetting->ipaDestination.networkaddr, 8) == 0);
	    bSourcePortOk = (strncmp (pPakketje->ipaIPXSource.hostaddr, fsSetting->ipaSource.hostaddr, 12) == 0);
	    bDestPortOk =   (strncmp (pPakketje->ipaIPXDestination.hostaddr, fsSetting->ipaDestination.hostaddr, 12) == 0);
	    bSourceSockOk = (strncmp (pPakketje->ipaIPXSource.socketaddr, fsSetting->ipaSource.socketaddr, 4) == 0);
	    bDestSockOk =   (strncmp (pPakketje->ipaIPXDestination.socketaddr, fsSetting->ipaDestination.socketaddr, 4) == 0);
	    bProtOk =   (pPakketje->uiIPXPacketType == fsSetting->uiIPXProtocol);
            
	    iReturn = true;
	    if (fsSetting->bIPXProtocolSpec && !bProtOk)
	    {
	        iReturn = false;
	    }
            else
            if (fsSetting->bIPXSrcHostSpec && !bSourcePortOk)
            {
                iReturn = false;
            }
            else
            if (fsSetting->bIPXDestHostSpec && !bDestPortOk)
            {
                iReturn = false;
            }
            else
            if (fsSetting->bIPXSrcSocketSpec && !bSourceSockOk)
            {
                iReturn = false;
            }
            else
            if (fsSetting->bIPXDestSocketSpec && !bDestSockOk)
            {
                iReturn = false;
            }
            else
            if (fsSetting->bIPXSrcNetSpec && !bSourceOk)
            {
                iReturn = false;
            }
            else
            if (fsSetting->bIPXDestNetSpec && !bDestOk)
            {
                iReturn = false;
            }
            break;
	default:
	    printf ("Unknown setting type 0x%x!\n", fsSetting->iType);
	    break;
    }
    iReturn ? PrintWithCare (-1, "*") : PrintWithCare (-1, "-");
    fflush (stdout);
    return iReturn;
}

static boolean CheckStartTrigger(const struct trigger_setting* tsSetting, 
                                 const struct packet* pPakketje)
{
    int iReturn = false;
    bool bSourceOk;
    bool bDestOk;
    bool bProtOk;
    bool bSourcePortOk;
    bool bDestPortOk;
    bool bSourceSockOk;
    bool bDestSockOk;

    printf ("."); fflush (stdout);
    switch (tsSetting->iStartType)
    {
        case TB_GEEN:
	    iReturn = true;
	    break;
	case TB_TIJD_START:
	case TB_TIJD_START_STOP:
	    if (strncmp ((char *) &tsSetting->tBegin, (char *) &pPakketje->tBegin, 14) > 0)
	    {
                iReturn = false;
	    }
	    else
	    {
		iReturn = true;
	    }
	    break;
	case TB_ETHERNET:
	    bSourceOk = (strncmp (pPakketje->eaEthSource.addr, tsSetting->eaSource.addr, 12) == 0);
	    bDestOk =   (strncmp (pPakketje->eaEthDestination.addr, tsSetting->eaDestination.addr, 12) == 0);
	    bProtOk =   (pPakketje->iEthernetProtocol == tsSetting->iEthernetProtocol);
            
	    iReturn = true;
	    if (tsSetting->bEthernetProtocolSpec && !bProtOk)
	    {
	        iReturn = false;
	    }
            else
            if (tsSetting->bEthernetSrcSpec && !bSourceOk)
            {
                iReturn = false;
            }
            else
            if (tsSetting->bEthernetDestSpec && !bDestOk)
            {
                iReturn = false;
            }
            break;
	case TB_IP:
            if (pPakketje->iEthernetProtocol != P3N_IP)
            {
                iReturn = false;
                break;
            }
	    bSourceOk = (strncmp (pPakketje->iaIPSource.addr, tsSetting->iaSource.addr, 8) == 0);
	    bDestOk =   (strncmp (pPakketje->iaIPDestination.addr, tsSetting->iaDestination.addr, 8) == 0);
            bProtOk =   (pPakketje->bIPProtocol == tsSetting->iIPProtocol);
	    if (tsSetting->iIPProtocol == P4N_TCP)
	    {
	        bSourcePortOk = (tsSetting->iIPSrcPort == pPakketje->uiTCPSourcePort);
	        bDestPortOk = (tsSetting->iIPDestPort == pPakketje->uiTCPDestinationPort);
	    }
	    else
	    if (tsSetting->iIPProtocol == P4N_UDP)
	    {
	        bSourcePortOk = (tsSetting->iIPSrcPort == pPakketje->uiUDPSourcePort);
	        bDestPortOk = (tsSetting->iIPDestPort == pPakketje->uiUDPDestinationPort);
	    }
	    else
	    {
	        bSourcePortOk = true;
                bDestPortOk = true;
	    }
	    iReturn = true;
	    if (tsSetting->bIPSrcPortSpec && !bSourcePortOk)
	    {
	        iReturn = false;
	    }
            else
            if (tsSetting->bIPDestPortSpec && !bDestPortOk)
            {
                iReturn = false;
            }
            else
	    if (tsSetting->bIPProtocolSpec && !bProtOk)
	    {
	        iReturn = false;
	    }
            else
            if (tsSetting->bIPSrcSpec && !bSourceOk)
            {
                iReturn = false;
            }
            else
            if (tsSetting->bIPDestSpec && !bDestOk)
            {
                iReturn = false;
            }
            break;
	case TB_IPX:
            if (pPakketje->iEthernetProtocol != P3N_IPX)
            {
                iReturn = false;
                break;
            }

	    bSourceOk = (strncmp (pPakketje->ipaIPXSource.networkaddr, tsSetting->ipaSource.networkaddr, 8) == 0);
	    bDestOk =   (strncmp (pPakketje->ipaIPXDestination.networkaddr, tsSetting->ipaDestination.networkaddr, 8) == 0);
	    bSourcePortOk = (strncmp (pPakketje->ipaIPXSource.hostaddr, tsSetting->ipaSource.hostaddr, 12) == 0);
	    bDestPortOk =   (strncmp (pPakketje->ipaIPXDestination.hostaddr, tsSetting->ipaDestination.hostaddr, 12) == 0);
	    bSourceSockOk = (strncmp (pPakketje->ipaIPXSource.socketaddr, tsSetting->ipaSource.socketaddr, 4) == 0);
	    bDestSockOk =   (strncmp (pPakketje->ipaIPXDestination.socketaddr, tsSetting->ipaDestination.socketaddr, 4) == 0);
	    bProtOk =   (pPakketje->uiIPXPacketType == tsSetting->uiIPXProtocol);
            
	    iReturn = true;
	    if (tsSetting->bIPXProtocolSpec && !bProtOk)
	    {
	        iReturn = false;
	    }
            else
            if (tsSetting->bIPXSrcHostSpec && !bSourcePortOk)
            {
                iReturn = false;
            }
            else
            if (tsSetting->bIPXDestHostSpec && !bDestPortOk)
            {
                iReturn = false;
            }
            else
            if (tsSetting->bIPXSrcSocketSpec && !bSourceSockOk)
            {
                iReturn = false;
            }
            else
            if (tsSetting->bIPXDestSocketSpec && !bDestSockOk)
            {
                iReturn = false;
            }
            else
            if (tsSetting->bIPXSrcNetSpec && !bSourceOk)
            {
                iReturn = false;
            }
            else
            if (tsSetting->bIPXDestNetSpec && !bDestOk)
            {
                iReturn = false;
            }
            break;
	default:
	    break;
    }
    if (iReturn)
    {
        bCheckForTrigger = false;
	bTriggered = true;
        return true;
    }
    else
    {
        return false;
    }
}

static boolean CheckStopTrigger (const struct trigger_setting* tsSetting, 
                                 const struct packet* pPakketje)
{
    int iReturn = false;
    
    switch (tsSetting->iStopType)
    {
        case TS_GEEN:
	    iReturn = false;
	    break;
	case TS_TIJD:
	    if (strncmp ((char *) &tsSetting->tStop, (char *) &pPakketje->tBegin, 14) > 0)
	    {
                iReturn = false;
	    }
	    else
	    {
		iReturn = true;
	    }
	    break;
	case TS_AANTAL_PAKKET:
	    if (iFiltPack < tsSetting->iAantalPakketten)
	        iReturn = false;
	    else
	        iReturn = true;
	    break;
	default:
	    printf ("Unknown setting type 0x%x!\n", tsSetting->iStartType);
	    break;
    }
    return iReturn;
}

static void Cleanup(int signo)
{
    PcapPrintStatus ();	
    (void) Locking (LCK_REMOVE);
    TerminateDB ();
    exit (0);
}

void default_print (register const struct timeval *tvp, 
                    register const u_char *bp, register const u_int iLength, 
                    const u_int uLLC)
{
    register struct tm *gmt;
    struct tm sgmt;
    struct packet pPakketje; 
    int iT = 0;
    char sTimeStamp[21];
    boolean bOk;
	
    gmt = &sgmt;
    *gmt = *gmtime (&tvp->tv_sec);
    FormatTimeStamp (sTimeStamp, *gmt, tvp->tv_usec);

    for (iT = 0; iT < 20; iT++)
    {
        pPakketje.tBegin.cYear[iT] = sTimeStamp[iT];
    }
    ParsePacket (bp, &pPakketje, uLLC);
    
    iTotPack++;
    
    bOk =  (bCheckForTrigger && !CheckStartTrigger (&tsSetting, &pPakketje));
    if (bOk)
    {
    }
    else 
    {
        if (CheckFilter (&fsSetting, &pPakketje))
	{
            PutPacketData (&pPakketje);
	    iFiltPack++;
	    iFiltData += iLength;
	}
	iSnifPack++;
        iSnifData += iLength;
        bCheckForStopTrigger = true;
    }
    if (bCheckForStopTrigger && CheckStopTrigger (&tsSetting, &pPakketje))
    {
        bStopTrigger = true;
    }
}

void ether_if_print (u_char *user, const struct pcap_pkthdr *h, 
                     const u_char *p)
{
    u_int caplen = h->caplen;
    
    default_print (&h->ts, p, caplen, P2N_ETH);
}

void lo_if_print (u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
    u_int caplen = h->caplen;
    
    default_print (&h->ts, p, caplen, P2N_LO);
}

static int Locking (int iOpcode)
{
    FILE * fLock;
    int iPID;
    char sPID[10];
    
    iPID = getpid();
    sPID[5] = (iPID % 10) + 0x30;
    iPID /= 10;
    sPID[4] = (iPID % 10) + 0x30;
    iPID /= 10;
    sPID[3] = (iPID % 10) + 0x30;
    iPID /= 10;
    sPID[2] = (iPID % 10) + 0x30;
    iPID /= 10;
    sPID[1] = (iPID % 10) + 0x30;
    iPID /= 10;
    sPID[0] = (iPID % 10) + 0x30;
    sPID[6] = 0;

    switch (iOpcode)
    {
        case LCK_SET: fLock = fopen ("/var/lock/wesp", "w+");
	              fwrite (sPID, strlen (sPID), 1, fLock);
		      fclose (fLock);
		      break;
	case LCK_REMOVE: unlink ("/var/lock/wesp");
	                 break;
	case LCK_CHECK: fLock = fopen ("/var/lock/wesp", "r");
	                if (fLock == NULL)
			{
			    return LCK_NOT_LOCKED;
			}
			else
			{
			    return LCK_LOCKED;
			}
			break;
    }
    return 0;
}


void raw_if_print (u_char *user, const struct pcap_pkthdr *h, const u_char *p)
{
    u_int caplen = h->caplen;
    
    default_print (&h->ts, p, caplen, P2N_RAW);
}

static void Reload (int signo)
{
    printf ("\nReloading settings...\n");
    GetTriggerSetting ("default", &tsSetting);
    GetFilterSetting ("default", &fsSetting);
    bSleeping = false;
    bStopTrigger = true;
}

static void Usage(void)
{
    extern char version[];
    extern char pcap_version[];

    (void) fprintf (stderr, "%s version %s\n", program_name, version);
    (void) fprintf (stderr, "libpcap version %s\n", pcap_version);
    (void) fprintf (stderr, "Usage: wesp [-i interface]\n");
    exit (-1);
}

/* EOF */